'****************************************************************'
'*       _____   .__                    .___                    *'
'*      /     \  |__|  ____ _____     __| _/ ____  ___  ___     *'
'*     /  \ /  \ |  | / ___\\__  \   / __ |_/ __ \ \  \/  /     *'
'*    /    Y    \|  |/ /_/  >/ __ \_/ /_/ |\  ___/  >    <      *'
'*    \____|__  /|__|\___  /(____  /\____ | \___  >/__/\_ \     *'
'*            \/    /_____/      \/      \/     \/       \/     *'
'****************************************************************'
'*  Name    : MI0283QT2_par_v12.BAS                             *'
'*  Author  : Gabriel-Florin Mihaila                            *'
'*  Notice  : Copyright  Migadex 2011                          *'
'*  Date    : 08/08/2011                                        *'
'*  Version : 0.12                                              *'
'*  Notes   : MI0283QT TFT Display - Parallel Driver & example  *'
'*          : Graphic Library                                   *'
'****************************************************************'
;Optimiser_Level = 3                  ;     
;All_Digital = true

Include "Placa_18f87j10.inc"           ; board declares


Declare LCD_Type = Graphic            ; use this declaration as code make use of compiler's hijacked PRINT command

         

;==============================================================================
' Definition of the coded RGB 5-6-5 and some of the most used colors
; Note:
;       For  8 Bit colors R,G,B  3,3,2 bits
;       For 16 Bit colors R,G,B  5,6,5 bits
;==============================================================================
; define the RGB coding for 5 red | 6 green | 5 blue
$define RGB(r,g,b)      ((( r & $F8)<<8)|(( g & $FC)<<3)|(( b & $F8)>>3 ))   
; and now some colors
$define NONE            RGB($00, $40, $00)  ; use this as transparent color 
$define BLACK           RGB($00, $00, $00)   
$define WHITE           RGB($FF, $FF, $FF)  
$define RED             RGB($FF, $00, $00)
$define GREEN           RGB($00, $FF, $00)
$define BLUE            RGB($01, $00, $FF)
$define YELLOW          RGB($FF, $FF, $00)
$define MAGENTA         RGB($FF, $00, $FF)
$define CYAN            RGB($00, $FF, $FF)
$define GRAY            RGB($80, $80, $40)
$define SILVER          RGB($A0, $A0, $80)
$define GOLD            RGB($A0, $A0, $40)
$define ORANGE          RGB($FF, $6D, $00)
$define VIOLET          RGB($85, $00, $DF)
;==============================================================================

; color table bits defines
$ifndef COLOR_TABLE_BITS
    $define COLOR_TABLE_BITS     3                                  ; 2^2 = 4 colors in color table (minimum)
$endif
;
Dim COLOR_TABLE_SIZE As (2 << COLOR_TABLE_BITS)                     ; sizeof glcd_Colors[]
Dim glcd_Colors[4] As Word                                          ;
$define glcdBkColor                     glcd_Colors#0               ; background color
$define glcdFgColor                     glcd_Colors#1               ; foreground color
$define glcdFrColor                     glcd_Colors#2               ; frame color
$define glcdShColor                     glcd_Colors#3               ; shadow color
$define glcdSetColor(index, COLOR)      glcd_Colors[index]= COLOR   ; index : numeral only
$define glcdSetBkColor(COLOR)           glcdSetColor(0, COLOR)      ; background color
$define glcdSetFgColor(COLOR)           glcdSetColor(1, COLOR)      ; foreground color
$define glcdSetFrColor(COLOR)           glcdSetColor(2, COLOR)      ; frame color
$define glcdSetShColor(COLOR)           glcdSetColor(3, COLOR)      ; shadow color
$define glcdSetColors(fgColor, bkColor) glcdSetFgColor(fgColor): glcdSetBkColor(bkColor)
;==============================================================================

;==============================================================================
' Declaration of the S65 Driver Variables and Constants
;==============================================================================
;S65_Driver_Vars:
             ; used in writeSPI macro as return variable
    ;Dim __InByte            As Byte
    Dim dx                  As Dword        ;  
    Dim dy                  As Dword
    Dim dx2                 As Dword
    Dim dy2                 As Dword
    Dim err                 As Dword
    ;
    Dim errFill             As Dword        ; used in glcdFillCircle
    ;
    Dim stepx               As Dword        ;  
    Dim __reg               As Byte         ;
    Dim __param             As Byte         ;   
    Dim WREG_TEMP           As Byte   ; filling
    Dim _t                  As Byte  ;  
    Dim stepy               As Dword        ;    
    
    Dim Orient              As Word
    ; access
    Dim x0Fill              As Word         ; used in glcdFillCircle
    Dim y0Fill              As Word         ; used in glcdFillCircle
    Dim LCD_WIDTH           As Word
    Dim LCD_HEIGHT          As Word
    ;
    Dim glcd_cursor_xm      As Word         ; fill circle
    Dim glcd_cursor_ym      As Word
    ;
    Dim __Data              As Word
    Dim x0                  As Word
    Dim y0                  As Word
    Dim xp0                 As Word         ; in set pixel
    Dim yp0                 As Word         ;
    Dim x1                  As Word
    Dim y1                  As Word
    Dim xa0                 As Word
    Dim xa1                 As Word
    Dim ya0                 As Word
    Dim ya1                 As Word
    Dim xfr0                As Word         ; glcdDrawCMD rectangle
    Dim xfr1                As Word         ;
    Dim yfr0                As Word         ;
    Dim yfr1                As Word         ;
    Dim COLOR               As Word System
    Dim tmp                 As Word
    Dim Size                As Dword
    Dim Size_B              As Dword
    ;
    Dim _X                  As Word;Byte
    Dim _Y                  As Word;Byte
    Dim radius              As Word;Byte
    Dim parL                As Word
    Dim parR                As Word
    ;
    Dim _P1                 As Word         ; glcdFillCircle
    Dim _P2                 As Word
    Dim _P3                 As Word
    Dim _P4                 As Word
    ;
    Dim glcd_FontFirstChar  As Byte         ; first defined character in selected font
    Dim glcd_FontLastChar   As Byte         ; last defined character in selected font
    Dim glcd_FontWidth      As Byte         ; character width for fixed pitch of selected font
    Dim glcd_FontHeight     As Byte         ; character height of selected font
    Dim glcd_FontBitsPixel  As Byte         ; color depth of selected font, if MSB is set the font is compressed
    Dim glcd_FontSize       As Word         ; Font size
    Dim glcd_Clip_x1        As Word
    Dim glcd_Clip_x2        As Word
    Dim glcd_Clip_y1        As Word
    Dim glcd_Clip_y2        As Word
    Dim glcd_FontData       As  Word            ; STORES OUT THE START OF THE LOOKUP TABLE FOR THIS FONT 
    Dim FONT_NAME           As  glcd_FontData   ; Font Table Address
    ;
    Dim glcd_Cursor_X       As Word System
    Dim glcd_Cursor_Y       As Word System
    Dim glcd_Window_x1      As Word
    Dim glcd_Window_x2      As Word             ;
    Dim glcd_Window_y1      As Word
    Dim glcd_Window_y2      As Word
    ;
    Dim charwidth           As Byte 
    Dim width               As Byte
    Dim _i                  As Byte
    Dim bitspixel           As Byte
    Dim bitsmask            As Byte System
    Dim bits                As Word System            ;
    Dim bits_tWord          As Byte
    Dim bitscount           As Byte             ;
    Dim table[4]            As Byte             ;
    Dim padding             As Byte             ;
    Dim index               As Dword
    Dim index_byte          As Byte
    Dim pixelcount          As Byte             ;
    Dim clipinvalid         As Byte             ;
    Dim xx                  As Word             ; glcdCoord_t xx;
    Dim yy                  As Word             ; glcdCoord_t yy;
    Dim pixelcolor          As Word System            ; color 
    Dim clipY               As Word             ; glcdCoord_t type 
    Dim stopY               As Word             ;
    Dim stopX               As Word             ;
    Dim _charToPrint        As Byte System
    ;==============================================
    ; structure like glcd_FlagsBits
    Dim glcd_FlagsBits      As Byte         ; with the following structure bits
    $define AutoLineFeed 0                  ; Font routines should use implicit linefeed And wrap around glcd_Window
    $define FixedFont    1                  ; == 0 fonts painted proportional, == 1 fonts painted in fixed pitch
    $define Clipping     2                  ; activate clipping
    ;==============================================
    ;
    Dim intermed_index      As Word
    $define FONT_HEADER_SIZE 7
    ;
    Dim tblptrl_temp        As Word
    Dim tblptr              As TBLPTRL.Word
    ;  
    Dim GEN                 As Byte System        ; used with #disable PRINT
    Dim GENH                As Byte System
    Dim PP0                 As Byte System
    Dim PP0_temp            As Byte
    Dim PP0h                As Byte System
    Dim PP0h_temp           As Byte
    Dim PP0hh               As Byte System
    Dim PP0hh_temp          As Byte
    Dim PP0hhh              As Byte System
    Dim PP0hhh_temp         As Byte
    Dim PP2                 As Byte System
    Dim PP2_temp            As Byte
    Dim PP2h                As Byte System
    Dim PP2h_temp           As Byte
    Dim PP2hh               As Byte System
    Dim PP2hh_temp          As Byte
    Dim PP2hhh              As Byte System
    Dim PP2hhh_temp         As Byte
    Dim PP7                 As Byte System
    Dim PP7_temp            As Byte
    Dim PP7h                As Byte System
    Dim PP7h_temp           As Byte
    Dim PP7hh               As Byte System
    Dim PP7hh_temp          As Byte
    Dim PP7hhh              As Byte System
    Dim PP7hhh_temp         As Byte
    ;
    ; added for parallel interface
    Dim Bus_DataHi        As Byte  ; temp holder for datah on the 8bit LCD bus
    Dim Bus_DataLo        As Byte  ; temp holder for data  on the 8bit LCD bus
    Dim AdrSet_Temp       As Byte
;==============================================================================
;   User Variables
;==============================================================================
    Dim DEMO_word_A         As Word         ; user var
    Dim DEMO_word_B         As Word         ; user var
    Dim Reg_Index           As Byte System
    Dim Reg_Val             As Byte System
;==============================================================================    
    
    
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; T F T   M A P P I N G    -   O P T I O N S     S E C T I O N S
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;    Select the Screen Orientation (define and/or un-define the next declares
;                                   as per above schematic drawings ) 
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$define use_glcdSetOrientation              ; 
;$define use_glcdReset                       ;
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;    Define or Un-Define the graphical functions to be used
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$define use_drawLine                        ; user code make use of glcdDrawLine graphic function. Comment If Not needed
$define use_drawRectangle                   ; user code make use of glcdDrawRectangle graphic function. Comment If Not needed
$define use_drawCircle                      ; user code make use of glcdDrawCircle graphic function. Comment If Not needed
$define use_fillRectangle                   ; user code make use of glcdFillRectangle graphic function. Comment If Not needed
$define use_fillCircle                      ; user code make use of glcdFillCircle graphic function. Comment if not needed 
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;    Define or Un-Define the text/font related functions to be used
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$define use_AutoLineFeed                    ; enable this function if AutoLineFeed needed in user code 
$define use_glcdSelectFont                  ; enable different Font Selection in user code
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;    GLCD PARTICULAR COMMANDS
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
$define use_compilerPrint                   ; using compiler's PRINT keyword for S65 commands
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
;    Fonts to be used
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
; insert here your font files and define/un-define them as needed
$define use_Font_f8x8                       ; will use the font stored in file "f8x8.inc"
$define use_Font_clock24x49                 ; will use the Font stored in file "Clock24x49.inc"
$define use_Font_f13x16                     ; will use the Font stored in file "f13x16.inc" 
$define use_Font_f8x11                      ; will use the Font stored in file "f8x11.inc"
$define use_Font_f12x12                     ; will use the Font stored in file "f12x12.inc"  
$define use_Font_f9x14                      ; will use the Font stored in file "f9x14.inc"
;$define use_Font_nums                       ; will use the Font stored in file "nums.inc"
;$define use_Font_decode6x5                  ; will use the Font stored in file "decode6x5.inc"
$define use_Font_arrows15x18                ; will use the Font stored in file "arrows15x18.inc" 
;        
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++     


'####################     HARDWARE DEFINEs & MACROS ###########################





GoTo MMB_Board

#Disable Print           
Asm
Print    
    GoTo glcdDrawChar_SUB
EndAsm   




;==============================================================================

;    User defined Macro's       

'==============================================================================
glcdStrobe Macro
    GoSub _glcdStrobe
    Endm
_glcdStrobe:
    LCD_WR_PIN = 0
    LCD_WR_PIN = 1
    Return
;==============================================================================
 glcdWriteCmd Macro P1, P2                  ; P1 = register, P2 = val
    #if (Prm_1 == Byte)
		BYTE_BYTE P1, Reg_Index ;Bus_DataHi
	#endif
	#if (Prm_1 == Word)
	    WORD_BYTE P1, Reg_Index ;Bus_DataHi
    #endif
	#if (Prm_1 == Num8) || (Prm_1 == Num16)
		NUM_BYTE P1, Reg_Index ;Bus_DataHi
	#endif 
    #if (Prm_2 == Byte)
		BYTE_BYTE P2, Reg_Val ;Bus_DataLo
	#endif
	#if (Prm_2 == Word)
	    WORD_BYTE P2, Reg_Val ;Bus_DataLo
    #endif
	#if (Prm_2 == Num8) || (Prm_2 == Num16)
		NUM_BYTE P2, Reg_Val ;Bus_DataLo
	#endif 
    GoSub __glcdWriteCmd
    Endm
    
__glcdWriteCmd:
    LCD_CS_PIN = 0
    '
    LCD_RS_PIN = 0                    ;//
    LCD_DATA_PORT = Reg_Index ;Bus_DataLo             
    glcdStrobe
    '
    LCD_RS_PIN = 1
    LCD_DATA_PORT = Reg_Val
    glcdStrobe
    '
    LCD_CS_PIN = 1
    Return
;==============================================================================
glcdWriteData   Macro P1, P2
    #if (Prm_1 == Byte)
		BYTE_BYTE P1, Bus_DataHi
	#endif
	#if (Prm_1 == Word)
	    WORD_BYTE P1, Bus_DataHi
    #endif
	#if (Prm_1 == Num8) || (Prm_1 == Num16)
		NUM_BYTE P1, Bus_DataHi
	#endif 
    #if (Prm_2 == Byte)
		BYTE_BYTE P2, Bus_DataLo
	#endif
	#if (Prm_2 == Word)
	    WORD_BYTE P2, Bus_DataLo
    #endif
	#if (Prm_2 == Num8) || (Prm_2 == Num16)
		NUM_BYTE P2, Bus_DataLo
	#endif 
    GoSub __glcdWriteData
    Endm
    
__glcdWriteData:
    LCD_DATA_PORT = Bus_DataHi             
    glcdStrobe
    LCD_DATA_PORT = Bus_DataLo
    glcdStrobe
    Return
     
            
;==============================================================================
;    GLCD display mapping related Macro's       
;                     define a partial screen area on the display 
;                     that will be used for automatic printing
;==============================================================================
glcdSetArea     Macro P1, P2, P3, P4        ; x0, y0, x1, y1
            ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, xa0
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, xa0
	        #endif
            #if (Prm_1 == Num8) || (Prm_1 == Num16)         ;
		      NUM_WORD P1, xa0
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, ya0
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, ya0
	        #endif
            #if (Prm_2 == Num8) || (Prm_2 == Num16)         ;
		      NUM_WORD P2, ya0
	        #endif
	        ;
            #if (Prm_3 == Byte)             ; 320
		      BYTE_WORD P3, xa1
	        #endif
            #if (Prm_3 == Word)
		      WORD_WORD P3, xa1
	        #endif
            #if (Prm_3 == Num8) || (Prm_3 == Num16)         ;
		      NUM_WORD P3, xa1
	        #endif
	        ;
            #if (Prm_4 == Byte)             ; 320
		      BYTE_WORD P4, ya1
	        #endif
            #if (Prm_4 == Word)
		      WORD_WORD P4, ya1
	        #endif
            #if (Prm_4 == Num8) || (Prm_4 == Num16)         ;
		      NUM_WORD P4, ya1
	        #endif
            ;
            GoSub setPartialArea_SUB
            Endm
           
setPartialArea_SUB:            
      If xa1 >= LCD_WIDTH  Then Return
      If ya1 >= LCD_HEIGHT Then Return ' error check
      ;
      glcdWriteCmd $03, xa0.Byte0  ; set x0
      glcdWriteCmd $02, xa0.Byte1  ;
      glcdWriteCmd $05, xa1.Byte0  ; set x1
      glcdWriteCmd $04, xa1.Byte1  ;
      glcdWriteCmd $07, ya0.Byte0  ; set y0
      glcdWriteCmd $06, ya0.Byte1  ;
      glcdWriteCmd $09, ya1.Byte0  ; set y1
      glcdWriteCmd $08, ya1.Byte1  ;
      ;
      Return;

;==============================================================================
; set cursor to x, y coordinates on the display
;==============================================================================
glcdSetCursor   Macro P1, P2                    ; glcd_Cursor_X , glcd_Cursor_Y
	        ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, glcd_Cursor_X
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, glcd_Cursor_X
	        #endif
            #if (Prm_1 == Num8) || (Prm_1 == Num16)         ;
		      NUM_WORD P1, glcd_Cursor_X
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, glcd_Cursor_Y
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, glcd_Cursor_Y
	        #endif
            #if (Prm_2 == Num8) || (Prm_2 == Num16)         ;
		      NUM_WORD P2, glcd_Cursor_Y
	        #endif
	        ;
            GoSub setCursor_SUB
            Endm

setCursor_SUB:
            glcdSetArea  glcd_Cursor_X , glcd_Cursor_Y , glcd_Cursor_X , glcd_Cursor_Y
            
            Return;
;==============================================================================
glcdDrawStart   Macro
            GoSub drawStart_SUB
            Endm
drawStart_SUB:            
            LCD_CS_PIN = 0
            '
            LCD_RS_PIN = 0                    ;//
            LCD_DATA_PORT = $22 ;target GRAM             
            glcdStrobe
            '
            LCD_RS_PIN = 1                    ;//          
            Return;
;==============================================================================
glcdDrawCMD        Macro P1                          ; 16 bit color variable
	         ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Num8) || (Prm_1 == Num16)         ;
		      NUM_WORD P1, COLOR
	        #endif
	        ;
            GoSub drawCMD_SUB
            Endm
drawCMD_SUB:                                     ;
            glcdWriteData COLOR.Byte1 , COLOR.Byte0
            Return;
;==============================================================================
glcdDrawStop    Macro
            GoSub drawStop_SUB
            Endm
drawStop_SUB:            
            LCD_CS_PIN = 1 ;LCD_CS_DISABLE()                          ; 
            Return;
;==============================================================================
glcdClear   Macro P1                                 ; 16 bit color variable
            #if (Prm_1 == Byte)
		      byte_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Num8)  || (Prm_1 == Num16)
		      NUM_WORD P1, COLOR
	        #endif
            GoSub glcdClear_SUB
            Endm

glcdClear_SUB:
            glcdSetArea 0, 0, (LCD_WIDTH-1), (LCD_HEIGHT-1);           
            ;
            glcdDrawStart                                ;
            Size = LCD_WIDTH * LCD_HEIGHT
            ;
            Repeat
              glcdDrawCMD COLOR
              Dec Size
            Until Size = 0
            ;
            glcdDrawStop                                 ;
            Return;
;==============================================================================
glcdDrawPixel   Macro P1, P2, P3                     ;  xp0, yp0, color
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, xp0
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, xp0
	        #endif
            #if (Prm_1 == Num8) || (Prm_1 == Num16)         ;
		      NUM_WORD P1, xp0
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, yp0
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, yp0
	        #endif
            #if (Prm_2 == Num8) || (Prm_2 == Num16)         ;
		      NUM_WORD P2, yp0
	        #endif
	        ;
            #if (Prm_3 == Byte)             ; 320
		      BYTE_WORD P3, COLOR
	        #endif
            #if (Prm_3 == Word)
		      WORD_WORD P3, COLOR
	        #endif
            #if (Prm_3 == Num8) || (Prm_3 == Num16)         ;
		      NUM_WORD P3, COLOR
	        #endif
            ;
            GoSub drawPixel_SUB
            Endm
drawPixel_SUB:
            glcdSetArea xp0, yp0, xp0, yp0
            glcdDrawStart                                ;
            glcdDrawCMD COLOR                            ;
            glcdDrawStop                                 ;
            Return;
;==============================================================================
glcdPutPixel    Macro P1                                 ; P1 =  color
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, COLOR
	        #endif
            #if (Prm_1 == Num8) || (Prm_1 == Num16)         ;
		      NUM_WORD P1, COLOR
	        #endif
            GoSub putPixel_SUB
            Endm
putPixel_SUB:
            glcdDrawStart                                ;
            glcdDrawCMD COLOR                            ;
            glcdDrawStop                               ;
            Return;
;==============================================================================
$ifdef use_glcdSelectFont                  
glcdSelectFont      Macro   P1                                     ; P1 as selected font label name
                    #if (Prm_1 == LABEL)
		              label_word P1, FONT_NAME
	                #endif
                    GoSub glcdSelectFont_SUB
                    Endm
glcdSelectFont_SUB:
                    glcd_FontSize       = CRead  glcd_FontData + 0 ; font_Size_in_Bytes_over_all_included_Size_it_self;
                    glcd_FontWidth      = CRead  glcd_FontData + 2 ; font_Width_in_Pixel_for_fixed_drawing
                    glcd_FontHeight     = CRead  glcd_FontData + 3 ; font_Height_in_Pixel_for_all_Characters
                    glcd_FontBitsPixel  = CRead  glcd_FontData + 4 ; font_Bits_per_Pixels // if MSB are set then font is a compressed font
                    glcd_FontFirstChar  = CRead  glcd_FontData + 5 ; font_First_Char
                    glcd_FontLastChar   = CRead  glcd_FontData + 6 ; font_Last_Char
                    Return
$endif
;==============================================================================
; glcdNewLine
;==============================================================================                                    
$ifdef use_AutoLineFeed
glcdNewLine         Macro-                                          ; <CR><LF> in glcd_Window of glcd_Cursor
                    GoSub glcdNewLine_SUB
                    Endm
glcdNewLine_SUB:
                    glcd_Cursor_X = glcd_Window_x1               
                    glcd_Cursor_Y = glcd_Cursor_Y  + glcd_FontHeight 
                    If glcd_Cursor_Y + glcd_FontHeight > glcd_Window_y2 Then
                        glcd_Cursor_Y = glcd_Window_y1 
                    EndIf
                    Return
$endif
;==============================================================================
;  glcdDrawChar      ; glcdDrawCMD a character, returns width of char, if any painted
;==============================================================================          
glcdDrawChar    Macro   P1                       ; char  number to be printed   ie if "A" then = $41
                #if (Prm_1 == Num8) 
	               Num_byte P1,_charToPrint		 ; Point FSR to the start of the string
                #endif
                #if (Prm_1 == Byte) 
	               Byte_Byte P1,_charToPrint		 
                #endif
                #if (Prm_1 == WREG) 
	               WREG_byte _charToPrint 		 
                #endif
                GoSub glcdDrawChar_SUB
                Endm

glcdDrawChar_SUB:   
   
    $ifdef use_compilerPrint
        WREG_TEMP = WREG
        PP0_temp = PP0
        PP0h_temp = PP0h
        PP0hh_temp = PP0hh
        PP0hhh_temp = PP0hhh
        PP2_temp = PP2
        PP2h_temp = PP2h
        PP2hh_temp = PP2hh
        PP2hhh_temp = PP2hhh
        PP7_temp = PP7
        PP7h_temp = PP7h
        PP7hh_temp = PP7hh
        PP7hhh_temp = PP7hhh
    $endif
    
    _charToPrint = WREG
        
    If _charToPrint = 0 Then                     ; CR/LF
        glcdNewLine                              ;
        Return
    EndIf
  ; is the character defined in the font file and a valid font size selected ?
    If _charToPrint < glcd_FontFirstChar Or _charToPrint > glcd_FontLastChar Or glcd_FontSize = 0 Then
        Return;
    EndIf
  ; is the character in the font declared but not defined?
    (* charwidth = CRead glcd_FontData + FONT_HEADER_SIZE + _charToPrint - glcd_FontFirstChar ;*)
    intermed_index = _charToPrint - glcd_FontFirstChar
    charwidth = CRead glcd_FontData + FONT_HEADER_SIZE + intermed_index;
    ;
    If charwidth = 0 Then 
        Return;
    EndIf
  ; all characters have on their right side, a small implicit empty column
    width = charwidth;
    If _charToPrint < 128 Then 
       Inc width ; 
    EndIf
  ; if we are to draw the font with fixed width then set the width accordingly
    If glcd_FlagsBits.FixedFont = 1 And width <= glcd_FontWidth Then 
        width = glcd_FontWidth;
    EndIf
  ; adjusts the drawn characters in the character window adding a linefeed
    If glcd_FlagsBits.AutoLineFeed = 1 And (glcd_Cursor_X + width) >= glcd_Window_x2 Then
        glcdNewLine
    EndIf
  ; is the character positioned inside the visible area ?
    If glcd_Cursor_X > glcd_Clip_x2 Or glcd_Cursor_Y > glcd_Clip_y2 Then 
        Return
    EndIf
    xx = glcd_Cursor_X;  make a copy of actual cursor position
    yy = glcd_Cursor_Y;
    glcd_Cursor_X = glcd_Cursor_X + width                   ; Prepare next x coord for the next char on line
  ; is the character positioned inside the visible area ?
    If xx > glcd_Clip_x2 Or (yy + glcd_FontHeight) < glcd_Clip_y1 Then
        Return;
    EndIf
    _i = 0   
  ; To be completed . . . 
  ; If FixedFont must be centered within a rectangle, but only
  ; if the sign is smaller than glcd_FontWidth. We will also fill the same right
  ; and left edges of the character, only if the background color is not transparent.
    If glcd_FlagsBits.FixedFont = 1 And width > charwidth Then 
        _i = (width - charwidth) / 2 
        If glcdBkColor <> NONE Then  
        (*  glcdDoFillRect(xx, yy, xx + _i -1, yy + glcd_FontHeight -1, glcdBkColor   ;*)
        EndIf
        xx = xx + _i
    EndIf
    _i = width - charwidth - _i
'    If _i > 0 And glcdBkColor <> NONE Then 
'    (*  glcdDoFillRect(xx + charwidth, yy, xx + charwidth + _i -1, yy + glcd_FontHeight -1, glcdBkColor)   ;*)
'    EndIf
  ; the actual character can be drawn?
    If xx > glcd_Clip_x2 Or (xx + charwidth) < glcd_Clip_x1 Then
        Return 
    EndIf
    bitspixel = glcd_FontBitsPixel & $7F;             ; Bits per Pixel
    bitsmask = $FF >> (8 - bitspixel);                ; Bitmask for color
   ; All reads in the font memory are structured so that they all blocks
   ; Bytes carried out sequentially. This is important and very well optimized for example. I2C EEPROM.
    If glcd_FontBitsPixel.7 = 1 Then 
        ;==================
        ; COMPRESSED FONTS
        ;==================
        ; Get the length of the RLE BytePadding table and calculate the Byteindex to the data.
        ; This reading has an additional overhead of 4 bytes for compressed fonts.
        ; For compressed fonts no left clipping can be done  :(
        intermed_index = glcd_FontData + FONT_HEADER_SIZE
        intermed_index = intermed_index + glcd_FontLastChar - glcd_FontFirstChar
        intermed_index = intermed_index + 1
        padding  = CRead intermed_index ;(data++);
        Inc intermed_index              ; 
        table[0] = 1                    ;
        table[1] = CRead intermed_index ;(data++);
        Inc intermed_index
        table[2] = CRead intermed_index ;(data++);
        Inc intermed_index
        table[3] = CRead intermed_index ;(data++);
        Inc intermed_index
        index = 0                       ;
        _i = glcd_FontFirstChar
        While _i < _charToPrint
            index_byte = CRead intermed_index 
            index = index + index_byte
            Inc intermed_index
            Inc _i
        Wend
        index = index * padding;
        intermed_index = glcd_FontLastChar - glcd_FontFirstChar + 3
        intermed_index = intermed_index  * 2
        intermed_index = intermed_index  +  glcd_FontData + FONT_HEADER_SIZE
        intermed_index = intermed_index  + index
        bits = CRead intermed_index     ;(data++);
        bitscount = 8                   ;
    Else 
    ;====================
    ; UNCOMPRESSED FONTS
    ;====================
    ; now calculate the bit index to the bit data in the font
        intermed_index = glcd_FontData + FONT_HEADER_SIZE;
        index = 0;
        _i = glcd_FontFirstChar
        While _i < _charToPrint
             index_byte = CRead intermed_index ;(data++);  
             index = index + index_byte
             Inc intermed_index
             Inc _i
        Wend
     ; Left clipping into account, only possible with uncompressed fonts
        If xx < glcd_Clip_x1 Then 
            _t = glcd_Clip_x1 - xx   
            index = index + _t 
            charwidth = charwidth - _t     
            xx = glcd_Clip_x1 
        EndIf
        index = index * glcd_FontHeight *  bitspixel 
    ; would position the data's first byte font files with a sign and get the first bits
        bitscount = index // 8  
        index = index / 8
        intermed_index = glcd_FontData + FONT_HEADER_SIZE
        intermed_index = intermed_index + glcd_FontLastChar - glcd_FontFirstChar
        intermed_index = intermed_index  + 1 + index  
        bits = CRead intermed_index                 ; here we read the first bits of font data to display ;  
        bits = bits >> bitscount                    ;
        bitscount = 8 - bitscount                   ;
    EndIf
  ; we send the bit data of each pixel in the yy direction of each Column
    pixelcount = 0                                  ; Number of pixels to be set
    pixelcolor = 0                                  ; Pixel color to be Set 
    clipinvalid = 1                                 ; Validity of the address pointers on the GLCD
    clipY = 0                                   
    stopY = glcd_Cursor_Y + glcd_FontHeight -1      ; lowest pixel line of the character
    stopX = xx + charwidth -1 
  ; right clipping account
    If stopX > glcd_Clip_x2 Then 
        stopX = glcd_Clip_x2 
    EndIf
    ;
    For xx = xx To stopX   
        If clipY = clipY | glcd_Cursor_Y Then 
            clipinvalid = 1;
        EndIf
        For yy = glcd_Cursor_Y To stopY    
                If pixelcount = 0 Then 
                    If bitscount <= 8 Then 
                        Inc intermed_index
                        bits_tWord = CRead intermed_index  ; read a byte
                        bits = bits | (bits_tWord << bitscount)
                        bitscount = bitscount + 8 
                    EndIf
                    If glcd_FontBitsPixel.7 = 1 Then       ; if the character font is compressed 
                        pixelcount = table[bits & 3] 
                        bits = bits >> 2 
                        bitscount = bitscount -2 
                    Else 
                        Inc pixelcount 
                    EndIf
                    If bitspixel <= COLOR_TABLE_BITS Then 
                        pixelcolor = glcd_Colors[bits & bitsmask] 
                    Else 
                        pixelcolor = bits & bitsmask;
                    EndIf
                    bits = bits >> bitspixel;
                    bitscount = bitscount - bitspixel;
                EndIf
                Dec pixelcount  
                If pixelcolor <> NONE  And yy >= glcd_Clip_y1 And yy <= glcd_Clip_y2 And xx >= glcd_Clip_x1 Then 
                    glcdSetArea xx, yy, xx, yy
                    clipY = yy 
                    glcdPutPixel pixelcolor 
                Else 
                    clipinvalid = 1 
                EndIf
        Next
    Next
 $ifdef use_compilerPrint
        PP0 = PP0_temp
        PP0h = PP0h_temp
        PP0hh = PP0hh_temp
        PP0hhh = PP0hhh_temp
        PP2 = PP2_temp
        PP2h = PP2h_temp
        PP2hh = PP2hh_temp
        PP2hhh = PP2hhh_temp
        PP7 = PP7_temp
        PP7h = PP7h_temp
        PP7hh = PP7hh_temp
        PP7hhh = PP7hhh_temp
        WREG = WREG_TEMP
    $endif
    Return 
;==============================================================================
$ifdef use_fillRectangle
glcdFillRectangle    Macro   P1, P2, P3, P4, P5
	        ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, x0
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, x0
	        #endif
            #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
		      NUM_WORD P1, x0
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, y0
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, y0
	        #endif
            #if (Prm_2 == Num8) | (Prm_2 == Num16)         ;
		      NUM_WORD P2, y0
	        #endif
	        ;
            #if (Prm_3 == Byte)             ; 320
		      BYTE_WORD P3, x1
	        #endif
            #if (Prm_3 == Word)
		      WORD_WORD P3, x1
	        #endif
            #if (Prm_3 == Num8) | (Prm_3 == Num16)         ;
		      NUM_WORD P3, x1
	        #endif
	        ;
            #if (Prm_4 == Byte)             ; 320
		      BYTE_WORD P4, y1
	        #endif
            #if (Prm_4 == Word)
		      WORD_WORD P4, y1
	        #endif
            #if (Prm_4 == Num8) | (Prm_4 == Num16)         ;
		      NUM_WORD P4, y1
	        #endif
            ;
	        #if (Prm_5 == Byte)             ; 320
		      BYTE_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Word)
		      WORD_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Num8) | (Prm_5 == Num16)         ;
		      NUM_WORD P5, COLOR
	        #endif
            ;
	        GoSub fillRectangle_SUB
	        Endm

fillRectangle_SUB:
            If x0 > x1 Then
                tmp = x0            ;
                x0  = x1            ;
                x1  = tmp           ;
            EndIf
            If y0 > y1 Then
                tmp = y0            ;
                y0  = y1            ;
                y1  = tmp           ;
            EndIf
            If x1 >= LCD_WIDTH  Or y1 >= LCD_HEIGHT Then
                Return;
            EndIf
            glcdSetArea x0, y0, x1, y1  ;
            glcdDrawStart               ;
            ;
            Size   = 1 + ( x1 - x0 )
            Size_B = 1 + ( y1 - y0 )
            Size = Size * Size_B 
            ;
            While Size > 0
                glcdDrawCMD COLOR
                Size = Size - 1     ; dec size
            Wend
            glcdDrawStop
            Return;
$endif
;==============================================================================
$ifdef use_drawRectangle
glcdDrawRectangle   Macro-   P1, P2, P3, P4, P5
	        ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, xfr0
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, xfr0
	        #endif
            #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
		      NUM_WORD P1, xfr0
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, yfr0
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, yfr0
	        #endif
            #if (Prm_2 == Num8) | (Prm_2 == Num16)         ;
		      NUM_WORD P2, yfr0
	        #endif
	        ;
            #if (Prm_3 == Byte)             ; 320
		      BYTE_WORD P3, xfr1
	        #endif
            #if (Prm_3 == Word)
		      WORD_WORD P3, xfr1
	        #endif
            #if (Prm_3 == Num8) | (Prm_3 == Num16)         ;
		      NUM_WORD P3, xfr1
	        #endif
	        ;
            #if (Prm_4 == Byte)             ; 320
		      BYTE_WORD P4, yfr1
	        #endif
            #if (Prm_4 == Word)
		      WORD_WORD P4, yfr1
	        #endif
            #if (Prm_4 == Num8) | (Prm_4 == Num16)         ;
		      NUM_WORD P4, yfr1
	        #endif
            ;
	        #if (Prm_5 == Byte)             ; 320
		      BYTE_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Word)
		      WORD_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Num8) | (Prm_5 == Num16)         ;
		      NUM_WORD P5, COLOR
	        #endif
            ;        
	        GoSub drawRectangle_SUB
	        Endm
drawRectangle_SUB:	        
            glcdFillRectangle xfr0, yfr0, xfr0, yfr1, COLOR ;
            glcdFillRectangle xfr0, yfr1, xfr1, yfr1, COLOR ;
            glcdFillRectangle xfr1, yfr0, xfr1, yfr1, COLOR ;
            glcdFillRectangle xfr0, yfr0, xfr1, yfr0, COLOR ;
            Return;
$endif
;==============================================================================
$ifdef use_drawLine
glcdDrawLine    Macro P1, P2, P3, P4, P5    ;        x0. y0. x1. y1. color
	        ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, x0
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, x0
	        #endif
            #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
		      NUM_WORD P1, x0
	        #endif
	        ;
            #if (Prm_2 == Byte)             ; 320
		      BYTE_WORD P2, y0
	        #endif
            #if (Prm_2 == Word)
		      WORD_WORD P2, y0
	        #endif
            #if (Prm_2 == Num8) | (Prm_2 == Num16)         ;
		      NUM_WORD P2, y0
	        #endif
	        ;
            #if (Prm_3 == Byte)             ; 320
		      BYTE_WORD P3, x1
	        #endif
            #if (Prm_3 == Word)
		      WORD_WORD P3, x1
	        #endif
            #if (Prm_3 == Num8) | (Prm_3 == Num16)         ;
		      NUM_WORD P3, x1
	        #endif
	        ;
            #if (Prm_4 == Byte)             ; 320
		      BYTE_WORD P4, y1
	        #endif
            #if (Prm_4 == Word)
		      WORD_WORD P4, y1
	        #endif
            #if (Prm_4 == Num8) | (Prm_4 == Num16)         ;
		      NUM_WORD P4, y1
	        #endif
            ;
	        #if (Prm_5 == Byte)             ; 320
		      BYTE_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Word)
		      WORD_WORD P5, COLOR
	        #endif
            #if (Prm_5 == Num8) | (Prm_5 == Num16)         ;
		      NUM_WORD P5, COLOR
	        #endif
            ;	        
            GoSub drawLine_SUB
            Endm            
drawLine_SUB:
            If x0 = x1 Or y0 = y1 Then                       ; horizontal Or vertical Line
                glcdFillRectangle x0, y0, x1, y1, COLOR     ; 0k
            EndIf
            ; Else
            ; calculate direction
            dx = x1 - x0                        ;
            ; bug correction
            If x0>x1 Then
               dx = dx - 65536
            EndIf
            ;
            dy = y1 - y0                        ;
            ; bug correction
            If y0>y1 Then
               dy = dy - 65536
            EndIf
            ;
            If dx < 0 Then
              dx = Abs dx
              stepx = -1
            Else 
              stepx = 1                         ;  
            EndIf
            If dy < 0  Then
              dy = Abs dy                       ; 
              stepy = -1                        ;   
            Else   
              stepy = 1                         ; 
            EndIf
            dx2 = dx << 1                       ;
            dy2 = dy << 1                       ;
            ; draw  Line
            glcdDrawPixel x0, y0, COLOR             ;
            ;
            If dx > dy Then
                  err = dy2 - dx                ;
                  While x0 <> x1 
                         If err >= 0 Then
                          y0  = y0 + stepy      ;
                          err = err - dx2       ;
                         EndIf
                         x0  = x0 + stepx       ;
                         err = err + dy2        ;
                         glcdDrawPixel x0, y0, COLOR  
                  Wend
            Else
                  err = dx2 - dy                ;
                  While y0 <> y1 
                         If err >= 0 Then
                          x0  = x0 + stepx      ;
                          err = err - dy2       ;
                         EndIf
                         y0  = y0 + stepy       ;
                         err = err + dx2        ;
                         glcdDrawPixel x0, y0, COLOR  
                  Wend
            EndIf
            Return;
$endif
;==============================================================================       
$ifdef use_drawCircle
glcdDrawCircle  Macro   p1, p2, p3, p4          ;  X0, Y0, RADIUS, color
        	        ;
                    #if (Prm_1 == Byte)             ; 320
        		      BYTE_WORD P1, x0
        	        #endif
                    #if (Prm_1 == Word)
        		      WORD_WORD P1, x0
        	        #endif
                    #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
        		      NUM_WORD P1, x0
        	        #endif
        	        ;
                    #if (Prm_2 == Byte)             ; 320
        		      BYTE_WORD P2, y0
        	        #endif
                    #if (Prm_2 == Word)
        		      WORD_WORD P2, y0
        	        #endif
                    #if (Prm_2 == Num8) | (Prm_2 == Num16)         ;
        		      NUM_WORD P2, y0
        	        #endif
        	        ;
        	        #if (Prm_3 == Byte)
        		      BYTE_BYTE P3, radius
        	        #endif
                    #if (Prm_3 == Num8)  ;| (Prm_1 == Num16)
        		      NUM_BYTE P3, radius
        	        #endif
        	        ;
        	        #if (Prm_4 == Byte)             ; 320
        		      BYTE_WORD P4, COLOR
        	        #endif
                    #if (Prm_4 == Word)
        		      WORD_WORD P4, COLOR
        	        #endif
                    #if (Prm_4 == Num8) | (Prm_4 == Num16)         ;
        		      NUM_WORD P4, COLOR
        	        #endif
                    ;	
                    GoSub drawCircle_SUB
                    Endm
 drawCircle_SUB:
                    err = - radius 
                    _X   = radius 
                    _Y   = 0 
                    While _X >= _Y 
                        parL = x0 + _X
                        parR = y0 + _Y
                        glcdDrawPixel parL , parR, COLOR ;1
                        parL = x0 - _X
                        parR = y0 + _Y
                        glcdDrawPixel parL , parR, COLOR ;2
                        parL = x0 + _X
                        parR = y0 - _Y
                        glcdDrawPixel parL , parR, COLOR ;3
                        parL = x0 - _X
                        parR = y0 - _Y
                        glcdDrawPixel parL , parR, COLOR ;4
                        parL = x0 + _Y
                        parR = y0 + _X
                        glcdDrawPixel parL , parR, COLOR ;5
                        parL = x0 - _Y
                        parR = y0 + _X
                        glcdDrawPixel parL , parR, COLOR ;6
                        parL = x0 + _Y
                        parR = y0 - _X
                        glcdDrawPixel parL , parR, COLOR ;7
                        parL = x0 - _Y
                        parR = y0 - _X
                        glcdDrawPixel parL , parR, COLOR ;8
                        err = err + _Y
                        _Y = _Y + 1
                        err = err + _Y
                        If err >= 0 Then
                          _X = _X - 1
                          err = err - _X
                          err = err - _X
                        EndIf
                    Wend
                    Return;
$endif  
                
;==============================================================================
$ifdef use_fillCircle
glcdFillCircle  Macro   P1, P2, P3, P4          ;   x0, y0, radius, color
        	        ;
                    #if (Prm_1 == Byte)             ; 320
        		      BYTE_WORD P1, x0Fill
        	        #endif
                    #if (Prm_1 == Word)
        		      WORD_WORD P1, x0Fill
        	        #endif
                    #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
        		      NUM_WORD P1, x0Fill
        	        #endif
        	        ;
                    #if (Prm_2 == Byte)             ; 320
        		      BYTE_WORD P2, y0Fill
        	        #endif
                    #if (Prm_2 == Word)
        		      WORD_WORD P2, y0Fill
        	        #endif
                    #if (Prm_2 == Num8) | (Prm_2 == Num16)         ;
        		      NUM_WORD P2, y0Fill
        	        #endif
        	        ;
        	        #if (Prm_3 == Byte)
        		      BYTE_BYTE P3, radius
        	        #endif
                    #if (Prm_3 == Num8)  ;| (Prm_1 == Num16)
        		      NUM_BYTE P3, radius
        	        #endif
        	        ;
        	        #if (Prm_4 == Byte)             ; 320
        		      BYTE_WORD P4, COLOR
        	        #endif
                    #if (Prm_4 == Word)
        		      WORD_WORD P4, COLOR
        	        #endif
                    #if (Prm_4 == Num8) | (Prm_4 == Num16)         ;
        		      NUM_WORD P4, COLOR
        	        #endif
                    ;
                    GoSub fillCircle_SUB
                    Endm
fillCircle_SUB:                     
                    errFill  = - radius;
                    glcd_cursor_xm   = radius;
                    glcd_cursor_ym   = 0;         
                    While glcd_cursor_xm >= glcd_cursor_ym 
                        _P1 = x0Fill - glcd_cursor_xm
                        _P2 = y0Fill + glcd_cursor_ym
                        _P3 = x0Fill + glcd_cursor_xm
                        _P4 = y0Fill + glcd_cursor_ym
                        glcdDrawLine _P1, _P2, _P3, _P4, COLOR;
                        _P1 = x0Fill - glcd_cursor_xm
                        _P2 = y0Fill - glcd_cursor_ym
                        _P3 = x0Fill + glcd_cursor_xm
                        _P4 = y0Fill - glcd_cursor_ym
                        glcdDrawLine _P1, _P2, _P3, _P4, COLOR
                        _P1 = x0Fill - glcd_cursor_ym
                        _P2 = y0Fill + glcd_cursor_xm
                        _P3 = x0Fill + glcd_cursor_ym
                        _P4 = y0Fill + glcd_cursor_xm
                        glcdDrawLine _P1, _P2, _P3, _P4, COLOR
                        _P1 = x0Fill - glcd_cursor_ym
                        _P2 = y0Fill - glcd_cursor_xm
                        _P3 = x0Fill + glcd_cursor_ym
                        _P4 = y0Fill - glcd_cursor_xm
                        glcdDrawLine _P1, _P2, _P3, _P4, COLOR
                        errFill = errFill + glcd_cursor_ym          ;
                        glcd_cursor_ym = glcd_cursor_ym + 1         ;
                        errFill = errFill + glcd_cursor_ym          ;
                        If errFill >= 0 Then
                          glcd_cursor_xm =  glcd_cursor_xm - 1
                          errFill = errFill - glcd_cursor_xm        ;
                          errFill = errFill - glcd_cursor_xm        ;
                        EndIf
                    Wend
                    Return;
$endif
;==============================================================================
$ifdef use_glcdSetOrientation
glcdSetOrientation Macro P1     ; orientation parameter in degrees. Valid values 0,90,180,270
          ;
            #if (Prm_1 == Byte)             ; 320
		      BYTE_WORD P1, Orient
	        #endif
            #if (Prm_1 == Word)
		      WORD_WORD P1, Orient
	        #endif
            #if (Prm_1 == Num8) | (Prm_1 == Num16)         ;
		      NUM_WORD P1, Orient
	        #endif
	        ;
            GoSub glcdSetOrientation_SUB
            Endm
glcdSetOrientation_SUB:
            Select Orient
                   Case 0
                        ;lcd_orientation = 0
                        LCD_WIDTH = 320
                        LCD_HEIGHT = 240
                        glcdWriteCmd $16, $A8 ; MY=1 MX=0 MV=1 ML=0 BGR=1
                   ;
                   Case 90
                        ;lcd_orientation = 90
                        LCD_WIDTH = 240
                        LCD_HEIGHT = 320
                        glcdWriteCmd $16, $08 ; MY=0 MX=0 MV=0 ML=0 BGR=1
                   ;
                   Case 180
                        ;lcd_orientation = 180
                        LCD_WIDTH = 320
                        LCD_HEIGHT = 240
                        glcdWriteCmd $16, $68 ; MY=0 MX=1 MV=1 ML=0 BGR=1     
                   ;
                   Case 270
                        ;lcd_orientation = 270
                        LCD_WIDTH = 240
                        LCD_HEIGHT = 320
                        glcdWriteCmd $16, $C8 ; MY=1 MX=0 MV=1 ML=0 BGR=1     
            EndSelect
            
            glcdSetArea 0, 0, (LCD_WIDTH - 1), (LCD_HEIGHT - 1)
            
              
            Return            
;==============================================================================
glcdReset Macro
          GoSub glcdReset_SUB
          Endm
          ;
glcdReset_SUB:
            LCD_RST_PIN = 0
            LCD_RS_PIN = 1
            LCD_CS_PIN = 0
            LCD_RD_PIN = 1
            LCD_WR_PIN = 1
            Clear LCD_DATA_PORT
            DelayMS 100
            LCD_RST_PIN = 1
            DelayMS 200
          
          
            ; driving ability
            glcdWriteCmd	$EA	,	$0
            glcdWriteCmd	$EB	,	$20
            glcdWriteCmd	$EC	,	$0C
            glcdWriteCmd	$ED	,	$C4
            glcdWriteCmd	$E8	,	$40
            glcdWriteCmd	$E9	,	$38
            glcdWriteCmd	$F1	,	$01
            glcdWriteCmd	$F2	,	$10
            glcdWriteCmd	$27	,	$A3
            glcdWriteCmd	$40	,	$00
            glcdWriteCmd	$41	,	$00
            glcdWriteCmd	$42	,	$01
            glcdWriteCmd	$43	,	$13
            glcdWriteCmd	$44	,	$10
            glcdWriteCmd	$45	,	$26
            glcdWriteCmd	$46	,	$08
            glcdWriteCmd	$47	,	$51
            glcdWriteCmd	$48	,	$02
            glcdWriteCmd	$49	,	$12
            glcdWriteCmd	$4A	,	$18
            glcdWriteCmd	$4B	,	$19
            glcdWriteCmd	$4C	,	$14
            glcdWriteCmd	$50	,	$19
            glcdWriteCmd	$51	,	$2F
            glcdWriteCmd	$52	,	$2C
            glcdWriteCmd	$53	,	$3E
            glcdWriteCmd	$54	,	$3F
            glcdWriteCmd	$55	,	$3F
            glcdWriteCmd	$56	,	$2E
            glcdWriteCmd	$57	,	$77
            glcdWriteCmd	$58	,	$0A
            glcdWriteCmd	$59	,	$06
            glcdWriteCmd	$5A	,	$07
            glcdWriteCmd	$5B	,	$0D
            glcdWriteCmd	$5C	,	$1D
            glcdWriteCmd	$5D	,	$CC
            			    
            glcdWriteCmd	$4	,	$00
            glcdWriteCmd	$5	,	$EF
            glcdWriteCmd	$8	,	$01
            glcdWriteCmd	$9	,	$3F
            			
            glcdWriteCmd	$16	,	$08
            glcdWriteCmd	$16	,	$68
            glcdWriteCmd	$1B	,	$1B
            glcdWriteCmd	$1A	,	$01
            glcdWriteCmd	$24	,	$2F
            glcdWriteCmd	$25	,	$57
            glcdWriteCmd	$23	,	$8D
            glcdWriteCmd	$18	,	$36
            glcdWriteCmd	$19	,	$01
            glcdWriteCmd	$1	,	$09
            glcdWriteCmd	$1F	,	$88
            DelayMS 5			
            glcdWriteCmd	$1F	,	$80
            DelayMS 5			
            glcdWriteCmd	$1F	,	$90
            DelayMS 5			
            glcdWriteCmd	$1F	,	$D0
            DelayMS 5			
            glcdWriteCmd	$17	,	$05
            glcdWriteCmd	$36	,	$00
            glcdWriteCmd	$28	,	$38
            DelayMS 40			
            glcdWriteCmd	$28	,	$3C
            ; 
            Return          
;-------------------------------------------------------------------------------











;==============================================================================                          
; Mikroe MBM Board - pins setup   (... yours could be different ...)
;==============================================================================
MMB_Board: 
  Clear  
  ;
  Clear PORTA
  Clear PORTB
  Clear PORTC
  Clear PORTD
  Clear LATD
  Clear PORTE
  Clear PORTF
  Clear PORTG
  Clear PORTH
  Clear PORTJ
  ;
  TRISA = 0
  TRISB = 0
  TRISC = 0
  TRISD = 0
  TRISE = 0
  TRISF = 0
  TRISG = 0
  TRISH = 0
  TRISJ = 0
  ;
  DelayMS 100
  ;
  ; any other needed declares for your specific board
;==============================================================================








;==============================================================================
; TFT INITIALIZATION
;==============================================================================
TFT_INIT: 
  glcdReset
;==============================================================================
; TFT display orientation degrees options
;==============================================================================
  glcdSetOrientation 0        ; 0, 90, 180 or 270
;==============================================================================
; by here we have a ready to use display
;==============================================================================




';==============================================================================
'; HERE GOES THE USER CODE
'; in this case it is an example of using fonts and graphic commands
';==============================================================================
MAIN_USER:
  ; define and set screen area
  ; set text colors
  glcdSetBkColor(BLACK)     ; background color
  glcdSetFgColor(GREEN)     ; foreground color
  glcdSetFrColor(GOLD)      ; frame color
  glcdSetShColor(SILVER)    ; shadow color
  ;
  glcd_FlagsBits.0 = 1      ; auto-feedline enabled
  glcd_FlagsBits.1 = 0      ; == 0 fonts painted proportional, == 1 fonts painted in fixed pitch
  glcd_FlagsBits.2 = 0      ; activate clipping
  ;
  glcd_Clip_x1 = 0          ; set it in init
  glcd_Clip_x2 = 319        ;
  glcd_Clip_y1 = 0          ;
  glcd_Clip_y2 = 239        ;
  ;
  glcd_Window_x1 = 0        ;
  glcd_Window_x2 = 319      ;
  glcd_Window_y1 = 0        ;
  glcd_Window_y2 = 239      ;
  ;  
  glcdClear BLACK           ; set initial screen background
  ;
  DelayMS 100 
  ;



DRAW_LINES:  
  
  glcdDrawLine 0,120,300,120, BLUE  ; ok
  DelayMS 100
  
  glcdDrawLine 160,5,160,230, BLUE  ; ok
  DelayMS 100
  
  glcdDrawLine 5,230,300,5, GREEN  ; ok
  DelayMS 100
  
  glcdDrawLine 5,5,300,230, GREEN  ; ok
  DelayMS 100 
  
  glcdFillRectangle 4,4,40,80, BLUE 
  DelayMS 100
  
  glcdDrawRectangle 4,21,160,220, SILVER
  DelayMS 100
  
  glcdDrawRectangle 125,81,173,129, RED
  DelayMS 100
  
  glcdDrawCircle 141,65,32, GREEN ;ok
  DelayMS 100
  
  glcdFillCircle 182,100,50, VIOLET
  DelayMS 100
  
  glcdDrawRectangle 0,0,319,239, ORANGE
  DelayMS 1000
  ; 
  
  glcdSelectFont f12x12   ; set the font to be used
  glcdSetFgColor(RED)     ; set the foreground color of the font
  glcdSetCursor 50,50
  Print "TESTING"
  
  glcdSelectFont f9x14    ; set the font to be used
  glcdSetFgColor(GREEN)   ; set the foreground color of the font
  glcdSetCursor 180,180
  Print "2233"            ;  

  ;
  Dim DEMO_BYTE As Byte
  glcdSetFgColor(GREEN)
  For DEMO_BYTE = 65 To 72
      glcdSelectFont arrows15x18
      glcdSetCursor 137,56
      Print DEMO_BYTE
      DelayMS 50
      glcdSelectFont arrows15x18
      glcdSetCursor 137,56
      Print " "
  Next 
  ; 
  DelayMS 500;1000
 
  
 

DRAW_TEXT:  
  DelayMS 500
  glcdClear BLACK          ; set the background color for this screen

  glcdSelectFont f13x16    ; set the font to be used
  glcdSetFgColor(RED)      ; set the font color
  glcdSetCursor 100,0      ; position the cursor
  Print "MI0283QT-2 GLCD"  ; print the string
   
  ; auto-feedline enabled example
  glcdSelectFont f8x8
  glcdSetFgColor(GREEN)
  glcdSetCursor 0,230
  Print "Privacy policy  amicus18.ro 2011                                     All rights reserved - v2.3 "

  glcdSelectFont f9x14 ; f12x12 ; f13x16 ;f9x14
  glcdSetFgColor(VIOLET)
  glcdSetCursor 60,40
  Print "PROTON PDS - Graphics Library"

  ; using multi-color fonts
  glcdSelectFont CLOCK24X49
  glcdSetFgColor(ORANGE)    ; numbers edge color , using specific color defined initialy
                            ; or    
  ; glcdSetFgColor(RGB($FF, $6D, $00))    ; numbers edge color , using the RGB HEX notation as with any color picker
  ;
  glcdSetShColor(RED)       ; numbers main color
  glcdSetFrColor(YELLOW)    ; inner filling color
  
COUNTING:
  glcd_FlagsBits.1 = 1      ; == 0 fonts painted proportional, == 1 fonts painted in fixed pitch
  For DEMO_word_B = 52421 To 52430
    glcdSetCursor 80,95
    Print Dec DEMO_word_B
    DelayMS 100
  Next
  glcd_FlagsBits.1 = 0

  DelayMS 300
  glcdClear RED
  DelayMS 100
  glcdClear GREEN
  DelayMS 100
  glcdClear BLUE
  DelayMS 100
  glcdClear BLACK
  DelayMS 100

;===========



  GoTo DRAW_LINES   ; and back to demo code
  
  
  
  





















;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; include FONTS, ICONS, IMAGES section   ; aici sint fonturile , etc
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 
$ifdef use_Font_f8x8
    Include "f8x8.inc"
$endif
$ifdef use_Font_clock24x49
    Include "Clock24x49.inc"
$endif
$ifdef use_Font_f13x16
    Include "f13x16.inc"
$endif    
$ifdef use_Font_f8x11
    Include "f8x11.inc"    
$endif
$ifdef use_Font_f12x12
    Include "f12x12.inc"
$endif    
$ifdef use_Font_f9x14     
    Include "f9x14.inc"
$endif
$ifdef use_Font_nums     
    Include "nums.inc"
$endif
$ifdef use_Font_decode6x5   
    Include "decode6x5.inc"
$endif    
$ifdef use_Font_arrows15x18   
    Include "arrows15x18.inc"
$endif

; Add more fonts here
    
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>    

  
